@@ -2,222 +2,45 @@ import json
 import logging
 import uuid
 from itertools import islice
-import fiona
-import fiona.transform
+
 from django.contrib.auth.models import Group
 from django.contrib.gis.db import models
 from django.contrib.gis.db.models.aggregates import Extent
-from django.contrib.gis.db.models.fields import BaseSpatialField
 from django.contrib.gis.db.models.functions import Transform
-from django.contrib.gis.geos import GEOSException, GEOSGeometry
+from django.contrib.gis.geos import GEOSGeometry, WKBWriter
 from django.contrib.gis.measure import D
 
+from .import_export.exports import LayerExportMixin
+from .import_export.imports import LayerImportMixin
+
 try:
     from django.db.models import JSONField
 except ImportError:  # TODO Remove when dropping Django releases < 3.1
     from django.contrib.postgres.fields import JSONField
-from django.contrib.gis.geos import WKBWriter
 from django.contrib.postgres.indexes import GistIndex, GinIndex
 from django.db import connection, transaction
 from django.db.models import Manager
-from django.db.models.signals import post_save
 from django.utils.functional import cached_property
 from django.utils.text import slugify
 from django.utils.translation import gettext_lazy as _
 
-from . import GeometryTypes  # NOQA
 from . import settings as app_settings
 from .db.managers import FeatureQuerySet
 from .db.mixins import BaseUpdatableModel, LayerBasedModelMixin
-from .functions import IsEmpty as IsEmptyFunc
-from .helpers import ChunkIterator
 from .routing.mixins import PgRoutingMixin, UpdateRoutingMixin
 
-from .signals import save_feature, save_layer_relation
-from .exports.helpers import generate_geojson, generate_shapefile
-from .tiles.decorators import zoom_update
 from .tiles.funcs import HausdorffDistance
 from .validators import (validate_geom_type, validate_json_schema,
                          validate_json_schema_data)
 
 logger = logging.getLogger(__name__)
 
-ACCEPTED_PROJECTIONS = [
-    'urn:ogc:def:crs:OGC:1.3:CRS84',
-    'EPSG:4326',
-]
 
-
-class Layer(LayerBasedModelMixin, UpdateRoutingMixin):
-    name = models.CharField(max_length=256, unique=True, default=uuid.uuid4)
-    schema = JSONField(default=dict, blank=True, validators=[validate_json_schema])
-    authorized_groups = models.ManyToManyField(Group, blank=True, related_name='authorized_layers')
-
-    def _initial_import_from_csv(self, chunks, options, operations):
-        for chunk in chunks:
-            entries = []
-            for row in chunk:
-                feature_args = {
-                    "geom": None,
-                    "properties": row,
-                    "layer": self
-                }
-
-                for operation in operations:
-                    operation(feature_args, options)
-
-                if not feature_args.get("geom"):
-                    logger.warning('empty geometry,'
-                                   f' row skipped : {row}')
-                    continue
-
-                entries.append(
-                    Feature(**feature_args)
-                )
-            Feature.objects.bulk_create(entries)
-
-    def _complementary_import_from_csv(self, chunks, options, operations,
-                                       pk_properties, fast=False):
-        for chunk in chunks:
-            sp = None
-            if fast:
-                sp = transaction.savepoint()
-            for row in chunk:
-                self._import_row_from_csv(row, pk_properties, operations,
-                                          options)
-            if sp:
-                transaction.savepoint_commit(sp)
-
-    def _import_row_from_csv(self, row, pk_properties, operations, options):
-        feature_args = {
-            "geom": None,
-            "properties": row,
-            "layer": self
-        }
-        for operation in operations:
-            operation(feature_args, options)
-        filter_kwargs = {
-            f'properties__{p}': feature_args["properties"].get(p, '')
-            for p in pk_properties}
-        filter_kwargs['layer'] = feature_args.get("layer", self)
-        if feature_args.get("geom"):
-            Feature.objects.update_or_create(
-                defaults=feature_args,
-                **filter_kwargs
-            )
-        else:
-            Feature.objects.filter(**filter_kwargs)\
-                .update(properties=feature_args["properties"])
-
-    @zoom_update
-    def from_csv_dictreader(self, reader, pk_properties, options, operations,
-                            init=False, chunk_size=1000, fast=False):
-        """Import (create or update) features from csv.DictReader object
-        :param reader: csv.DictReader object
-        :param pk_properties: keys of row that is used to identify unicity
-        :param init: allow to speed up import if there is only new Feature's
-                    (no updates)
-        :param chunk_size: only used if init=True, control the size of
-                           bulk_create
-        """
-        chunks = ChunkIterator(reader, chunk_size)
-        if init:
-            self._initial_import_from_csv(
-                chunks=chunks,
-                options=options,
-                operations=operations
-            )
-        else:
-            self._complementary_import_from_csv(
-                chunks=chunks,
-                options=options,
-                operations=operations,
-                pk_properties=pk_properties,
-                fast=fast
-            )
-
-    @zoom_update
-    def from_geojson(self, geojson_data, id_field=None, update=False):
-        """
-        Import geojson raw data in a layer
-        Args:
-            geojson_data(str): must be raw text json data
-        """
-        geojson = json.loads(geojson_data)
-        projection = geojson.get('crs', {}).get(
-            'properties', {}).get('name', None)
-        if projection and not self.is_projection_allowed(projection):
-            raise GEOSException(
-                f'GeoJSON projection {projection} must be in '
-                f'{ACCEPTED_PROJECTIONS}')
-
-        if update:
-            self.features.all().delete()
-        for feature in geojson.get('features', []):
-            properties = feature.get('properties', {})
-            identifier = properties.get(id_field, uuid.uuid4())
-            Feature.objects.update_or_create(
-                layer=self,
-                identifier=identifier,
-                defaults={
-                    'properties': properties,
-                    'geom': GEOSGeometry(json.dumps(feature.get('geometry'))),
-                }
-            )
-
-    def to_geojson(self):
-        return json.loads(generate_geojson(self))
-
-    def to_shapefile(self):
-        return generate_shapefile(self)
-
-    def _fiona_shape_projection(self, shape):
-        ''' Return projection in EPSG format or raw Proj format extracted from
-            shape
-        '''
-        projection = shape.crs
-        if projection and (len(projection) == 1 or
-           (len(projection) == 2 and projection.get('no_defs') is True)):
-            return projection.get('init')
-        else:
-            return fiona.crs.to_string(projection)
-
-    @zoom_update
-    def from_shapefile(self, zipped_shapefile_file, id_field=None):
-        ''' Load ShapeFile content provided into a zipped archive.
-
-        zipped_shapefile_file -- a file-like object on the zipped content
-        id_field -- the field name used a identifier
-        '''
-        with fiona.BytesCollection(zipped_shapefile_file.read()) as shape:
-            # Extract source projection and compute if reprojection is required
-            projection = self._fiona_shape_projection(shape)
-            reproject = projection and \
-                not self.is_projection_allowed(projection.upper())
-
-            for feature in shape:
-                properties = {}
-                for prop, value in feature.get('properties', {}).items():
-                    try:
-                        properties[prop] = json.loads(value)
-                    except (json.JSONDecodeError, TypeError):
-                        properties[prop] = value
-
-                geometry = feature.get('geometry')
-
-                if reproject:
-                    geometry = fiona.transform.transform_geom(
-                        shape.crs,
-                        f'EPSG:{app_settings.INTERNAL_GEOMETRY_SRID}',
-                        geometry)
-                identifier = properties.get(id_field, uuid.uuid4())
-
-                Feature.objects.create(
-                    layer=self,
-                    identifier=identifier,
-                    properties=properties,
-                    geom=GEOSGeometry(json.dumps(geometry)),
-                )
+class Layer(LayerBasedModelMixin, LayerImportMixin, LayerExportMixin, UpdateRoutingMixin):
+    name = models.CharField(max_length=256, unique=True, default=uuid.uuid4, verbose_name=_("Name"))
+    schema = JSONField(default=dict, blank=True, validators=[validate_json_schema], verbose_name=_("Schema"))
+    authorized_groups = models.ManyToManyField(Group, blank=True, related_name='authorized_layers',
+                                               verbose_name=_("Authorized groups"))
 
     @transaction.atomic
     def update_geometries(self, features):
@@ -252,6 +75,7 @@ class Layer(LayerBasedModelMixin, UpdateRoutingMixin):
             layer_field = Feature._meta.get_field('layer').get_attname_column()[1]
 
             cursor = connection.cursor()
+            # noinspection SqlResolve
             raw_query = f"""
                 SELECT
                     jsonb_object_keys(properties) AS key
@@ -291,9 +115,6 @@ class Layer(LayerBasedModelMixin, UpdateRoutingMixin):
 
         return prop_type
 
-    def is_projection_allowed(self, projection):
-        return projection in ACCEPTED_PROJECTIONS
-
     def get_extent(self, srid=3857):
         return self.features.annotate(
             geom_transformed=Transform('geom', srid)
@@ -301,6 +122,15 @@ class Layer(LayerBasedModelMixin, UpdateRoutingMixin):
             extent=Extent('geom_transformed')
         )
 
+    def get_property_values(self, property_to_list):
+        property_field = f'properties__{property_to_list}'
+
+        return (
+            self.features.order_by(property_field)
+            .values_list(property_field, flat='true')
+            .distinct(property_field)
+        )
+
     def __str__(self):
         return f"{self.name}"
 
@@ -314,9 +144,13 @@ class Layer(LayerBasedModelMixin, UpdateRoutingMixin):
 
 
 class LayerGroup(BaseUpdatableModel):
-    name = models.CharField(max_length=256, unique=True)
+    name = models.CharField(max_length=256,
+                            unique=True,
+                            verbose_name=_("Name"))
     slug = models.SlugField(unique=True)
-    layers = models.ManyToManyField(Layer, related_name='layer_groups')
+    layers = models.ManyToManyField(Layer,
+                                    related_name='layer_groups',
+                                    verbose_name=_("Layers"))
 
     def save(self, **kwargs):
         if self.pk is None:
@@ -329,12 +163,16 @@ class Feature(BaseUpdatableModel, PgRoutingMixin):
     identifier = models.CharField(max_length=255,
                                   blank=False,
                                   null=False,
-                                  default=uuid.uuid4)
-    properties = JSONField(default=dict, blank=True)
+                                  default=uuid.uuid4,
+                                  verbose_name=_("Identifier"))
+    properties = JSONField(default=dict,
+                           blank=True,
+                           verbose_name=_("Properties"))
     layer = models.ForeignKey(Layer,
                               on_delete=models.PROTECT,
                               related_name='features',
-                              db_index=False)
+                              db_index=False,
+                              verbose_name=_("Layer"))
 
     objects = Manager.from_queryset(FeatureQuerySet)()
 
@@ -441,9 +279,6 @@ class Feature(BaseUpdatableModel, PgRoutingMixin):
         ]
 
 
-post_save.connect(save_feature, sender=Feature)
-
-
 class LayerRelation(models.Model):
     RELATION_TYPES = (
         (None, 'Manual'),
@@ -478,31 +313,39 @@ class LayerRelation(models.Model):
         )
 
 
-post_save.connect(save_layer_relation, sender=LayerRelation)
-
-
 class FeatureRelation(models.Model):
     origin = models.ForeignKey(Feature,
                                on_delete=models.CASCADE,
-                               related_name='relations_as_origin')
+                               related_name='relations_as_origin',
+                               verbose_name=_("Origin"))
     destination = models.ForeignKey(Feature,
                                     on_delete=models.CASCADE,
-                                    related_name='relations_as_destination')
+                                    related_name='relations_as_destination',
+                                    verbose_name=_("Destination"))
     relation = models.ForeignKey(LayerRelation,
                                  on_delete=models.CASCADE,
-                                 related_name='related_features')
-    properties = JSONField(default=dict, blank=True)
+                                 related_name='related_features',
+                                 verbose_name=_("Relation"))
+    properties = JSONField(default=dict,
+                           blank=True,
+                           verbose_name=_("Properties"))
 
     class Meta:
         ordering = ['id']
 
 
 class LayerExtraGeom(LayerBasedModelMixin):
-    layer = models.ForeignKey(Layer, on_delete=models.CASCADE, related_name='extra_geometries')
-    order = models.PositiveSmallIntegerField(default=0)
+    layer = models.ForeignKey(Layer,
+                              on_delete=models.CASCADE,
+                              related_name='extra_geometries',
+                              verbose_name=_("Layer"))
+    order = models.PositiveSmallIntegerField(default=0,
+                                             verbose_name=_("Order"))
     slug = models.SlugField(editable=False)
-    title = models.CharField(max_length=250)
-    editable = models.BooleanField(default=True)
+    title = models.CharField(max_length=250,
+                             verbose_name=_("Title"))
+    editable = models.BooleanField(default=True,
+                                   verbose_name=_("Editable"))
 
     @cached_property
     def name(self):
@@ -527,11 +370,24 @@ class LayerExtraGeom(LayerBasedModelMixin):
 
 
 class FeatureExtraGeom(BaseUpdatableModel):
-    feature = models.ForeignKey(Feature, on_delete=models.CASCADE, related_name='extra_geometries')
-    layer_extra_geom = models.ForeignKey(LayerExtraGeom, on_delete=models.CASCADE, related_name='features')
-    geom = models.GeometryField(srid=app_settings.INTERNAL_GEOMETRY_SRID, spatial_index=False)
-    properties = JSONField(default=dict, blank=True)
-    identifier = models.UUIDField(blank=True, null=True, editable=False, default=uuid.uuid4)
+    feature = models.ForeignKey(Feature,
+                                on_delete=models.CASCADE,
+                                related_name='extra_geometries',
+                                verbose_name=_("Feature"))
+    layer_extra_geom = models.ForeignKey(LayerExtraGeom,
+                                         on_delete=models.CASCADE,
+                                         related_name='features',
+                                         verbose_name=_("Feature"))
+    geom = models.GeometryField(srid=app_settings.INTERNAL_GEOMETRY_SRID,
+                                spatial_index=False)
+    properties = JSONField(default=dict,
+                           blank=True,
+                           verbose_name=_("Properties"))
+    identifier = models.UUIDField(blank=True,
+                                  null=True,
+                                  editable=False,
+                                  default=uuid.uuid4,
+                                  verbose_name=_("Identifier"))
 
     class Meta:
         unique_together = (
@@ -542,8 +398,3 @@ class FeatureExtraGeom(BaseUpdatableModel):
             GistIndex(name='feg_geom_gist_index', fields=['layer_extra_geom', 'geom']),
             GinIndex(name='feg_properties_gin_index', fields=['properties']),
         ]
-
-
-@BaseSpatialField.register_lookup
-class IsEmpty(IsEmptyFunc):
-    lookup_name = 'isempty'
